//
// $Source: /mnt/cvs/rep/CW/HID08/Sources/usb_descriptors.c,v $
// $Id: usb_descriptors.c,v 1.1 2008/04/27 13:44:12 noritan Exp $
//
#include <hidef.h>
#include "derivative.h"
#include "usb_lowlevel.h"
#include "usb_descriptors.h"

//----------------------------------------------------------------------------
// My Own Report Descriptor
//   Generic 8-byte IN/OUT descriptor
//----------------------------------------------------------------------------
const byte reportDesc0[25] = {
    0x06, 0x00, 0xff,              // USAGE_PAGE (Vendor Defined Page 1)
    0x09, 0x01,                    // USAGE (Vendor Usage 1)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x95, 0x08,                    //   REPORT_COUNT (8)
    0x09, 0x01,                    //   USAGE (Vendor Usage 1)
    0x81, 0x02,                    //   INPUT (Data,Var,Abs)
    0x09, 0x01,                    //   USAGE (Vendor Usage 1)
    0x91, 0x02,                    //   OUTPUT (Data,Var,Abs)
    0xc0                           // END_COLLECTION
};

//----------------------------------------------------------------------------
// Keyboard Report Descriptor
//   Keyboard without LED
//----------------------------------------------------------------------------
const byte reportDesc1[45] = {
    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x09, 0x06,                    // USAGE (Keyboard)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x05, 0x07,                    //   USAGE_PAGE (Keyboard)
    0x19, 0xe0,                    //   USAGE_MINIMUM (Keyboard LeftControl)
    0x29, 0xe7,                    //   USAGE_MAXIMUM (Keyboard Right GUI)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
    0x75, 0x01,                    //   REPORT_SIZE (1)
    0x95, 0x08,                    //   REPORT_COUNT (8)
    0x81, 0x02,                    //   INPUT (Data,Var,Abs)
    0x95, 0x01,                    //   REPORT_COUNT (1)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x81, 0x03,                    //   INPUT (Cnst,Var,Abs)
    0x95, 0x06,                    //   REPORT_COUNT (6)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x25, 0x65,                    //   LOGICAL_MAXIMUM (101)
    0x05, 0x07,                    //   USAGE_PAGE (Keyboard)
    0x19, 0x00,                    //   USAGE_MINIMUM (Reserved (no event indicated))
    0x29, 0x65,                    //   USAGE_MAXIMUM (Keyboard Application)
    0x81, 0x00,                    //   INPUT (Data,Ary,Abs)
    0xc0                           // END_COLLECTION
};

//----------------------------------------------------------------------------
//  Device Descriptor Declaration
//----------------------------------------------------------------------------
const DeviceDescriptor deviceDesc = {
                    // Size of this Descriptor in Bytes
  sizeof(DeviceDescriptor),
  DT_DEVICE,        // Descriptor Type (=1)
  {0x00, 0x02},     // USB Spec Release Number in BCD = 2.00
  0,                // Device Class Code (none)
  0,                // Device Subclass Code	(none)
  0,                // Device Protocol Code (none)
  8,                // Maximum Packet Size for EP0 
  {0x00, 0x01},     // Vendor ID = 0x0100
  {0x00, 0x00},     // Product ID = 0
  {0x00, 0x00},     // Device Release Number in BCD
  1,                // Index of String Desc for Manufacturer
  2,                // Index of String Desc for Product
  0,                // Index of String Desc for SerNo
  1                 // Number of possible Configurations
};                  // end of deviceDesc

//----------------------------------------------------------------------------
//  Configuration descriptors
//  according to HID1.11 spec page 48
//----------------------------------------------------------------------------
const ConfigurationDescriptors configurationDescriptors = {

//----------------------------------------------------------------------------
//  Configuration Descriptor
//----------------------------------------------------------------------------
  {	            // Size of this Descriptor in Bytes
    sizeof(ConfigurationDescriptor),
    DT_CONFIGURATION,   // Descriptor Type (=2)
    {
      sizeof(ConfigurationDescriptor) +
      sizeof(InterfaceDescriptor) * 2 +
      sizeof(HidDescriptor) * 2 +
      sizeof(EndpointDescriptor) * 3,
      0x00},    // Total Length of Data for this Conf
    2,          // No of Interfaces supported by this Conf
    1,          // Designator Value for *this* Configuration
    0,          // Index of String Desc for this Conf
    0xc0,       // Self-powered, no Remote-Wakeup
    1           // Max. Power Consumption in this Conf (*2mA)
  },            // end of __configDesc

//----------------------------------------------------------------------------
//  Interface Descriptor
//----------------------------------------------------------------------------
  {             // Size of this Descriptor in Bytes
    sizeof(InterfaceDescriptor),
    DT_INTERFACE,   // Descriptor Type (=4)
    0,          // Number of *this* Interface (0..)
    0,          // Alternative for this Interface (if any)
    2,          // No of EPs used by this IF (excl. EP0)
    0x3,        // IF Class Code (0x03 - HID)
    0,          // No Interface Subclass Code
    0,				  // IF Protocol Code  (0 = None)
    0           // Index of String Desc for this Interface
  },            // end of __interfaceDesc0

//----------------------------------------------------------------------------
//  HID Descriptor #0
//----------------------------------------------------------------------------
  {	            // Size of this Descriptor in Bytes
    sizeof(HidDescriptor),
    DT_HID,    // Descriptor Type (=0x21)
    {0x01,0x01},    // HID class spec release 1.01
    0x00,      // Country code
    0x01,      // Number of descriptors
    DT_REPORT, // Descriptor type...?
               // Total size of report descriptor
    {sizeof(reportDesc0),0}
  },           // end of __hidDesc0
  
//----------------------------------------------------------------------------
//  IN-Endpoint #2 Descriptor
//----------------------------------------------------------------------------
  {             // Size of this Descriptor in Bytes
    sizeof(EndpointDescriptor),
    DT_ENDPOINT,    // Descriptor Type (=5)
    0x82,       // Endpoint Address (EP2, IN)
    0x03,       // Interrupt transfer
    {0x08, 0x00},   // Max. Endpoint Packet Size
    10          // Polling Interval (Interrupt) in ms
  },            // end of __endpointDesc2In
  
//----------------------------------------------------------------------------
//  OUT-Endpoint #2 Descriptor
//----------------------------------------------------------------------------
  {             // Size of this Descriptor in Bytes
    sizeof(EndpointDescriptor),
    DT_ENDPOINT,    // Descriptor Type (=5)
    0x02,       // Endpoint Address (EP2, OUT)
    0x03,       // Interrupt transfer
    {0x08, 0x00},   // Max. Endpoint Packet Size
    10          // Polling Interval (Interrupt) in ms
  },            // end of __endpointDesc2Out

//----------------------------------------------------------------------------
//  Interface Descriptor
//----------------------------------------------------------------------------
  {             // Size of this Descriptor in Bytes
    sizeof(InterfaceDescriptor),
    DT_INTERFACE,   // Descriptor Type (=4)
    1,          // Number of *this* Interface (0..)
    0,          // Alternative for this Interface (if any)
    1,          // No of EPs used by this IF (excl. EP0)
    0x3,        // IF Class Code (0x03 - HID)
    0,          // No Interface Subclass Code
    1,				  // IF Protocol Code  (1 = Keyboard)
    0           // Index of String Desc for this Interface
  },            // end of __interfaceDesc1

//----------------------------------------------------------------------------
//  HID Descriptor #1
//----------------------------------------------------------------------------
  {	            // Size of this Descriptor in Bytes
    sizeof(HidDescriptor),
    DT_HID,    // Descriptor Type (=0x21)
    {0x01,0x01},    // HID class spec release 1.01
    0x00,      // Country code
    0x01,      // Number of descriptors
    DT_REPORT, // Descriptor type...?
               // Total size of report descriptor
    {sizeof(reportDesc1),0}
  },           // end of __hidDesc1
  
//----------------------------------------------------------------------------
//  IN-Endpoint #2 Descriptor
//----------------------------------------------------------------------------
  {             // Size of this Descriptor in Bytes
    sizeof(EndpointDescriptor),
    DT_ENDPOINT,    // Descriptor Type (=5)
    0x81,       // Endpoint Address (EP1, IN)
    0x03,       // Interrupt transfer
    {0x08, 0x00},   // Max. Endpoint Packet Size
    10          // Polling Interval (Interrupt) in ms
  }             // end of __endpointDesc1In
  
};


//----------------------------------------------------------------------------
// String Descriptors
//   0: Language code
//   1: Manufacturer
//   2: Product
//   3: Interface (Not implemented)
//----------------------------------------------------------------------------

// Language IDs
const byte strDescLanguage[4] = {
  sizeof(strDescLanguage), DT_STRING,	      // Size, Type
  0x09, 0x04                                // LangID English (US)
};

// Manufacturer String
const byte strDescManufacturer [24] = {
  sizeof(strDescManufacturer), DT_STRING,   // Size, Type
  'n',0,'o',0,'r',0,'i',0,'t',0,
  'a',0,'n',0,'.',0,'o',0,'r',0,
  'g',0
};

// Product String
const byte strDescProduct [24] = {
  sizeof(strDescProduct), DT_STRING,        // Size, Type
  'L',0,'C',0,'D',0,' ',0,'D',0,
  'i',0,'s',0,'p',0,'l',0,'a',0,
  'y',0,
};

// Table of String Descriptors
const byte * const strDescTable[] = {
  strDescLanguage,
  strDescManufacturer,
  strDescProduct
};
const byte strDescTableSize = sizeof(strDescTable) / sizeof(*strDescTable);

//============================================================================
