//
// $Source: /mnt/cvs/rep/CW/HID08/Sources/usb_lowlevel.h,v $
// $Id: usb_lowlevel.h,v 1.1 2008/04/27 13:44:12 noritan Exp $
//
#ifndef __USBLOWLEVEL_H
#define __USBLOWLEVEL_H

//-- Data Type Definitions ---------------------------------------------------


//----------------------------------------------------------------------------
//  Intel type WORD description.
//----------------------------------------------------------------------------
typedef struct {                    // Data Type "Intel Word"
	  byte        lo;                 // (High/Low Byte swapped)
	  byte        hi;
}   iword;


//----------------------------------------------------------------------------
//  Standard Device Descriptor
//  according to USB2.0 spec page 262
//----------------------------------------------------------------------------
typedef struct {
    byte        bLength;            // Size of this Descriptor in Bytes
    byte        bDescriptorType;    // Descriptor Type (=1)
    iword       bcdUSB;             // USB Spec Release Number in BCD
    byte        bDeviceClass;       // Device Class Code
    byte        bDeviceSubClass;    // Device Subclass Code
    byte        bDeviceProtocol;    // Device Protocol Code
    byte        bMaxPacketSize0;    // Maximum Packet Size for EP0
    iword       idVendor;           // Vendor ID
    iword       idProduct;          // Product ID
    iword       bcdDevice;          // Device Release Number in BCD
    byte        iManufacturer;      // Index of String Desc for Manufacturer
    byte        iProduct;           // Index of String Desc for Product
    byte        iSerialNumber;      // Index of String Desc for SerNo
    byte        bNumConfigurations; // Number of possible Configurations
}   DeviceDescriptor;


//----------------------------------------------------------------------------
//  Standard Configuration Descriptor
//  according to USB2.0 spec page 265
//----------------------------------------------------------------------------
typedef struct {
    byte        bLength;            // Size of this Descriptor in Bytes
    byte        bDescriptorType;    // Descriptor Type (=2)
    iword       wTotalLength;       // Total Length of Data for this Conf
    byte        bNumInterfaces;     // No of Interfaces supported by this Conf
    byte        bConfigurationValue;  // Designator Value for *this* Configuration
    byte        iConfiguration;     // Index of String Desc for this Conf
    byte        bmAttributes;       // Configuration Characteristics (see below)
    byte        bMaxPower;          // Max. Power Consumption in this Conf (*2mA)
}   ConfigurationDescriptor;


//----------------------------------------------------------------------------
//  Standard Interface Descriptor
//  according to USB2.0 spec page 268
//----------------------------------------------------------------------------
typedef struct {
    byte        bLength;            // Size of this Descriptor in Bytes
    byte        bDescriptorType;    // Descriptor Type (=4)
    byte        bInterfaceNumber;   // Number of *this* Interface (0..)
    byte        bAlternateSetting;  // Alternative for this Interface (if any)
    byte        bNumEndpoints;      // No of EPs used by this IF (excl. EP0)
    byte        bInterfaceClass;    // Interface Class Code
    byte        bInterfaceSubClass; // Interface Subclass Code
    byte        bInterfaceProtocol; // Interface Protocol Code
    byte        iInterface;         // Index of String Desc for this Interface
}   InterfaceDescriptor;


//----------------------------------------------------------------------------
//  Standard HID Descriptor
//  according to HID1.11 spec page 22
//----------------------------------------------------------------------------
typedef struct {
    byte        bLength;            // Size of this Descriptor in Bytes
    byte        bDescriptorType;    // Descriptor Type (=5)
    iword       bcdHID;             // HID Class specification release number
    byte        bCountryCode;       // Hardware target country
    byte        bNumDesriptors;     // Number of HID class descriptors to follow
    byte        bRepDescriptorType; // Report descriptor type
    iword       wDescriptorLength;  // Total length of Report descriptor
}   HidDescriptor;


//----------------------------------------------------------------------------
//  Standard Endpoint Descriptor
//  according to USB2.0 spec page 269
//----------------------------------------------------------------------------
typedef struct {
    byte        bLength;            // Size of this Descriptor in Bytes
    byte        bDescriptorType;    // Descriptor Type (=5)
    byte        bEndpointAddress;   // Endpoint Address (Number + Direction)
    byte        bmAttributes;       // Endpoint Attributes (Transfer Type)
    iword       wMaxPacketSize;     // Max. Endpoint Packet Size
    byte        bInterval;          // Polling Interval (Interrupt) in ms
}   EndpointDescriptor;


//----------------------------------------------------------------------------
//  Structure of Setup Packet sent during
//  SETUP Stage of Standard Device Requests
//  according to USB2.0 spec page 248
//----------------------------------------------------------------------------
typedef struct {
    byte        bmRequestType;      // Characteristics (Direction,Type,Recipient)
    byte        bRequest;           // Standard Request Code
    word        wValue;             // Value Field
    word        wIndex;             // Index or Offset Field
    word        wLength;            // Number of Bytes to transfer (Data Stage)
}   SetupBuffer;


//----------------------------------------------------------------------------
//  USB Standard Device Request Codes
//  according to USB2.0 spec page 251
//----------------------------------------------------------------------------
#define     GET_STATUS          (0)
#define     CLEAR_FEATURE       (1)
#define     SET_FEATURE         (3)
#define     SET_ADDRESS         (5)
#define     GET_DESCRIPTOR      (6)
#define     SET_DESCRIPTOR      (7)   // optional
#define     GET_CONFIGURATION   (8)
#define     SET_CONFIGURATION   (9)
#define     GET_INTERFACE       (10)
#define     SET_INTERFACE       (11)
#define     SYNCH_FRAME         (12)  // optional


//----------------------------------------------------------------------------
//  USB Class-Specific Request Codes
//  according to HID1.11 spec page 51
//----------------------------------------------------------------------------
#define     GET_REPORT          (1)   // mandatory
#define     GET_IDLE            (2)
#define     GET_PROTOCOL        (3)
#define     SET_REPORT          (9)
#define     SET_IDLE            (10)
#define     SET_PROTOCOL        (11)


//----------------------------------------------------------------------------
//  Pseudo Request Code
//    REQUEST_COMPLETE - not part of the Standard - just a Flag
//    to indicate that the recent Request has been finished
//----------------------------------------------------------------------------
#define     REQUEST_COMPLETE    (255)


//----------------------------------------------------------------------------
//  Standard Descriptor Types
//  according to USB2.0 spec page 251
//----------------------------------------------------------------------------
#define     DT_DEVICE           (1)
#define     DT_CONFIGURATION    (2)
#define     DT_STRING           (3)
#define     DT_INTERFACE        (4)
#define     DT_ENDPOINT         (5)
#define     DT_DEVICE_QUALIFIER             (6)
#define     DT_OTHER_SPEED_CONFIGURATION    (7)
#define     DT_INTERFACE_POWER  (8)


//----------------------------------------------------------------------------
//  Class-Specific Descriptor Types
//  according to HID1.11 spec page 49
//----------------------------------------------------------------------------
#define     DT_HID              (0x21)
#define     DT_REPORT           (0x22)
#define     DT_PHYSICAL         (0x23)


//----------------------------------------------------------------------------
//  USB Status Codes
//----------------------------------------------------------------------------
#define     US_ATTACHED         (0x00)  // (not used here)
#define     US_POWERED          (0x01)
#define     US_DEFAULT          (0x02)
#define     US_ADDRESSED        (0x03)
#define     US_CONFIGURED       (0x04)
#define     US_SUSPENDED        (0x80)


//----------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------
#define     RT_STANDARD         (0x00)
#define     RT_CLASS            (0x20)
#define     RT_ENDPOINT         (0x02)


//----------------------------------------------------------------------------
//  
//----------------------------------------------------------------------------
#define     ENDPOINT_HALT       (0x00)



//----------------------------------------------------------------------------
//  TX/RX Buffer declaration.
//----------------------------------------------------------------------------
#define   EP0_MAXPACKETSIZE     (8)
#define   EP0OUT_SIZE   (8)     // Byte count of RX0
extern volatile byte    ep0OutBuffer[EP0OUT_SIZE];
#define   EP2IN_SIZE    (8)     // Byte count of TX2
extern volatile byte    Tx2Buffer[EP2IN_SIZE];
#define   EP2OUT_SIZE   (8)     // Byte count of RX2
extern volatile byte    Rx2Buffer[EP2OUT_SIZE];



//----------------------------------------------------------------------------
//  Function Prototypes
//----------------------------------------------------------------------------
void    initUsb(void);
byte    getUSB(void);
void    putUSB(byte c);
void    isrUSB(void);

void    dispatch(void);

extern  byte usbState;

//============================================================================
#endif
